//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
//                                  BASIC TEST №1 (Sumple Serial) 
//                  (compatible with FLProg - visual programming system)
// Only the standard Arduino IDE I/O functions are used.
// May be useful for testing new controllers during software debugging.
//================================================================================================
//    1.1.Setting parameters, incl. test pins from controller parameters. Can be changed.
//    1.2.In each loop() loop, the "Task Manager" is executed.
//    1.3.Every second the project performance is calculated.
//    2.1.The console is initialized (after initialization, no time is spent on execution).
//    2.2.After the console is ready, the header with the controller parameters is displayed once,
//        including scanning the i2c bus. If there are no pull-up resistors on the i2c bus to Vcc
//        or i2c devices, possibly hanging due to the CORE feature.
//    3.1.The change in each cycle of the loop is output on the Control pin (for a logic analyzer).
//    3.2.Meander=500ms or 250ms(button=1) is output on the Led pin.
//    4.1.The console (port monitor) displays the operating time (sec) and ADC values (1 time per second),
// 		  name of the board, speed, pin numbers (1 time in 5 seconds);
//-------------------------------------------------------------------------------------------------
//    To configure the IDE to work with supported boards, instructions to the Readme.hpp.
//-------------------------------------------------------------------------------------------------
// ecoins@mail.ru 07.05.23
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//#include "flprogUtilites.h"   		//--Connect SDK FLProg;
#define  RT_HW_TEST_PERMIT_PINS       	//--To block work with pins, comment out the line;
#include "RT_HW_BASE.h"         		//--Connection of the base library;;
//=================================================================================================
//                                   Project Performance Options
//=================================================================================================
uint8_t vInit=0;                      	//--Parameter initialization flag;
uint32_t codeCustomBrd=0;             	//--Custom board ID(=204001) (Raspberry Pi Pico DNS.01);
uint8_t debugCnt=0;                   	//--Test counter;
uint8_t pinControl;                   	//--Pin control;
uint8_t pinLed;                       	//--Pin Led on Board;                   
uint8_t pinButton; uint8_t vButton=0; 	//--Pin Button on Board;        
uint8_t pinADC;    uint16_t vADC;     	//--Pin ADC;
//======================================SETUP()====================================================
void setup(){}
//=======================================LOOP()====================================================
void loop(){
//=================================================================================================
//									1.SYSTEM FUNCTIONS
//=================================================================================================
if(!vInit){vInit=1;  Serial.begin(115200);
pinControl    =RT_HW_Base.device.pin.control;   //--Set pin Control;
pinLed        =RT_HW_Base.device.pin.led;       //--Set pin Led on Board;
pinButton     =RT_HW_Base.device.pin.button;    //--Set pin Burtton;
pinADC        =RT_HW_Base.device.pin.adc;       //--Set pin ADC;  
#if defined(RT_HW_TEST_PERMIT_PINS)             //--Checking the permission to work with pins;
  if(pinControl!=255){pinMode(pinControl,OUTPUT);}
  if(pinLed!=255)    {pinMode(pinLed,OUTPUT);}
  if(pinButton!=255) {pinMode(pinButton,INPUT_PULLUP);}
#endif  
} 
//-------------------------------------------------------------------------------------------------
RT_HW_Base.sheduler();                     		//--Sheduler tasks;
//=================================================================================================
//						2.OUTPUT TO CONTROL PIN AND LED, ADC READING
//=================================================================================================
#if defined(RT_HW_TEST_PERMIT_PINS)             //--Checking the permission to work with pins;
//-------------------------------------------------------------------------------------------------
if(pinControl!=255){digitalWrite(pinControl,RT_HW_Base.shed.control);}        //--loop() output to control pin; 
if(RT_HW_Base.shed.frdm.num==5){if(pinButton!=255){vButton=!digitalRead(pinButton);}} //--Input button;
if(RT_HW_Base.shed.frdm.num==1){                                              //--Вывод на Led;
 if(pinLed!=255){if(vButton){digitalWrite(pinLed,RT_HW_Base.shed.blink250);}  //  	meander 500ms; 
                 else       {digitalWrite(pinLed,RT_HW_Base.shed.blink500);}}}//  	meander 500ms;
if(RT_HW_Base.shed.fast.num==2){if(pinADC!=255)   {vADC=analogRead(pinADC);}} //--Input ADC; 
//-------------------------------------------------------------------------------------------------
#endif  //---END if  defined(RT_HW_TEST_PERMIT_PINS)
//=================================================================================================
//								3.PERIODIC CONSOLE OUTPUT
//=================================================================================================
if(RT_HW_Base.shed.slow.num==1) {if(Serial){
  if(!debugCnt){Serial.print(String(F("IDE->")));    Serial.print(String(F(RT_HW_BOARD_NAME)));   //--Name of the selected Arduino IDE board;
                Serial.print(String(F("; Base->"))); Serial.print(String(F(RT_HW_BOARD_SELECT))); //--Name of the selected board description RT_HW_BASE.h;
                Serial.print(String(F("; Speed="))); Serial.print(RT_HW_Base.shed.cycle);         //--Speed;
                Serial.println(';');
                Serial.print(String(F("pins-> control,Led,Button,ADC="))); 
                Serial.print(pinControl); Serial.print(',');    //--output number pin Control;
                Serial.print(pinLed);     Serial.print(',');    //--output number pin Led on Board;
                Serial.print(pinButton);  Serial.print(',');    //--output number pin Button on Board;
                Serial.print(pinADC);     Serial.println(';');  //--output number pin ADC;           
                }    
                Serial.print(String(F("Time=")));  Serial.print(RT_HW_Base.shed.cntSec);//--Output running time; 
                Serial.print(String(F("; ADC="))); Serial.print(vADC);        			//--Value from the ADC pin in ADC units;
                Serial.println(';');
  if(++debugCnt>=5){debugCnt=0;}}
}
//=================================================================================================
}; //++++END loop()================================================================================
//=================================================================================================   
