#include "RT_HW_BASE.h"	 
//#################################################################################################
//
//							 					I2C [.cpp]
//
// codeErr: 0-ok, 1-too much data; 2-Nack to send an address; 3-Nack for data transfer; 4-another error i2c; 
// 5-invalid controller;           6-invalid bus; 7-invalid address(>=254);
// 8-bus not configured as master; 9-another error;
//#################################################################################################

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							 	1.CREATING AN OBJECT i2c for MCU
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//										1.STM32
//=================================================================================================
#if !defined(RT_HW_CREATED_I2C) && defined(RT_HW_CORE_STM32)
#define RT_HW_CREATED_I2C
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C0)
#define RT_HW_CREATED_I2C0
TwoWire rtWire0(RT_HW_I2C0_SDA, RT_HW_I2C0_SCL);
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C1)
#define RT_HW_CREATED_I2C1
TwoWire rtWire1(RT_HW_I2C1_SDA, RT_HW_I2C1_SCL);
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C2)
#define RT_HW_CREATED_I2C
TwoWire rtWire2(RT_HW_I2C2_SDA, RT_HW_I2C2_SCL);
#endif
//-------------------------------------------------------------------------------------------------
#endif
//=================================================================================================
//										2.RP2040
//=================================================================================================
#if !defined(RT_HW_CREATED_I2C) && defined(RT_HW_CORE_RP2040)	
#define RT_HW_CREATED_I2C
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C0)	
#define RT_HW_CREATED_I2C0
TwoWire rtWire0(i2c0, RT_HW_I2C0_SDA, RT_HW_I2C0_SCL);
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C1)
#define RT_HW_CREATED_I2C1
TwoWire rtWire1(i2c1, RT_HW_I2C1_SDA, RT_HW_I2C1_SCL);
#endif
//--------------------------------------------------------------------------------------------------
#endif
//=================================================================================================
//										3.ESP32
//=================================================================================================
#if !defined(RT_HW_CREATED_I2C) && defined(RT_HW_CORE_ESP32)	
#define RT_HW_CREATED_I2C
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C0)	
#define RT_HW_CREATED_I2C0
TwoWire rtWire0(0);
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C1)
#define RT_HW_CREATED_I2C1
TwoWire rtWire1(1);
#endif
//--------------------------------------------------------------------------------------------------
#endif
//=================================================================================================
//										4.ESP8266
//=================================================================================================
#if !defined(RT_HW_CREATED_I2C) && defined(RT_HW_CORE_ESP8266)	
#define RT_HW_CREATED_I2C
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C0)	
#define RT_HW_CREATED_I2C0
TwoWire rtWire0;
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C1) && !defined(NO_GLOBAL_INSTANCES) && !defined(NO_GLOBAL_TWOWIRE)
#define RT_HW_CREATED_I2C1
TwoWire rtWire1;
#endif
//--------------------------------------------------------------------------------------------------
#endif
//=================================================================================================
//										5.Other MCU
//=================================================================================================
#if !defined(RT_HW_CREATED_I2C)	
#define RT_HW_CREATED_I2C
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C0)	
#define RT_HW_CREATED_I2C0
TwoWire rtWire0;
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C1)
#define RT_HW_CREATED_I2C1
TwoWire rtWire1;
#endif
//--------------------------------------------------------------------------------------------------
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//									2.FUNCTION begin/endTransaction(), i2cSetClock
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//									2.1.i2cBeginTransmission(adr,bus)
//=================================================================================================
void     RT_HW_BASE:: i2cBeginTransmission(uint8_t adr, uint8_t bus){
if(i2cGetStatus(bus)!=1){return;};		//--Exit if bus is not master;
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C0)
if(bus==0){rtWire0.beginTransmission(adr);  return;};
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C1)
if(bus==1){rtWire1.beginTransmission(adr); return;};
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C2)
if(bus==2){rtWire2.beginTransmission(adr); return;};
#endif	
//-------------------------------------------------------------------------------------------------		
if(adr || bus) {return;}	//--For blocking warnings from C++;
};
//=================================================================================================
//									2.2.i2cEndTransmission(bus)
//=================================================================================================
uint8_t  RT_HW_BASE:: i2cEndTransmission(uint8_t bus){
if(i2cGetStatus(bus)!=1){return 5;};	//--Exit if bus is not master;
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_CREATED_I2C0)
if(bus==0){return rtWire0.endTransmission();};
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C1)
if(bus==1){return rtWire1.endTransmission();};
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C2)
if(bus==2){return rtWire2.endTransmission();};
#endif
//-------------------------------------------------------------------------------------------------	
if(bus) {return 6;}		//--For blocking warnings from C++
return 6;}
//=================================================================================================
//						 			2.3.Set speed  bus i2c;
//=================================================================================================
void     RT_HW_BASE:: i2cSetClock(uint32_t speed,uint8_t bus){
if(speed==0){speed=RT_HW_I2C_SPEED;}
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C0) 
if(bus==0){rtWire0.setClock(speed);    return;}
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C1)
if(bus==1){rtWire1.setClock(speed); return;};
#endif
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C2)
if(bus==2){rtWire2.setClock(speed); return;}
#endif	
//-------------------------------------------------------------------------------------------------	
if(speed || bus) {return;}		//--For blocking warnings from C++;
};
//=================================================================================================


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//									3.FUNCTION i2c() 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//									3.1.i2cFlush(bus)
//=================================================================================================
uint8_t  RT_HW_BASE:: i2cFlush(uint8_t bus){
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C0)
if(bus==0){rtWire0.flush(); return 1;}
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C1)
if(bus==1){rtWire1.flush(); return 1;}
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C2)
if(bus==2){rtWire2.flush(); return 1;}
#endif	
//-------------------------------------------------------------------------------------------------	
if(bus) {return 0;}		//--For blocking warnings from C++
return 0;
};
//=================================================================================================
//									3.2.i2cAvailable(bus)
//=================================================================================================
uint8_t  RT_HW_BASE:: i2cAvailable(uint8_t bus){
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C0)
if(bus==0){return rtWire0.available();};
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C1)
if(bus==1){return rtWire1.available();};
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C2)
if(bus==2){return rtWire2.available();};
#endif
//-------------------------------------------------------------------------------------------------				
if(bus) {return 0;}		//--For blocking warnings from C++;
return 0;}
//=================================================================================================
//									3.2.i2cRead(bus)
//=================================================================================================
uint8_t  RT_HW_BASE:: i2cRead(uint8_t bus){
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C0)
if(bus==0){return rtWire0.read();};
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C1)
if(bus==1){return rtWire1.read();};
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C2)
if(bus==2){return rtWire2.read();};
#endif
//-------------------------------------------------------------------------------------------------				
if(bus) {return 0;}		//--For blocking warnings from C++;
return 0;};
//=================================================================================================
//									3.3.1. i2cWrite(var,bus)
//=================================================================================================
void     RT_HW_BASE:: i2cWrite(uint8_t var, uint8_t bus){
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C0)
if(bus==0){rtWire0.write(var); return;};
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C1)
if(bus==1){rtWire1.write(var); return;};
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C2)
if(bus==2){rtWire2.write(var); return;};
#endif
//-------------------------------------------------------------------------------------------------		
if(bus || var) {return;}		//--For blocking warnings from C++;
};
//=================================================================================================
//									3.3.2. i2cWrite(*arr, qnt, bus)
//=================================================================================================
void     RT_HW_BASE:: i2cWrite(const uint8_t *arr, uint8_t qnt, uint8_t bus){	
 if(qnt>32){qnt=32;} for(uint8_t i=0; i<qnt; i++){i2cWrite(arr[i],bus);}};	
//=================================================================================================
//									3.3.3. i2cWrite(*arr, qnt, bus)
//=================================================================================================
void     RT_HW_BASE:: i2cWrite(const char    *arr, uint8_t qnt, uint8_t bus){	//++++Запись qnt символов------
 if(qnt>32){qnt=32;} for(uint8_t i=0; i<qnt; i++){i2cWrite(arr[i],bus);}};	
//=================================================================================================
//									3.4. i2cRequestFrom(*arr, qnt, bus)
//=================================================================================================
uint8_t  RT_HW_BASE:: i2cRequestFrom(uint8_t qnt,  uint8_t adr, uint8_t bus, uint8_t sendStop){
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C0)
if(bus==0){return rtWire0.requestFrom(adr,qnt,(uint8_t)sendStop);};
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C1)
if(bus==1){return rtWire1.requestFrom(adr,qnt,(uint8_t)sendStop);};
#endif	
//-------------------------------------------------------------------------------------------------	
#if defined(RT_HW_CREATED_I2C2)
if(bus==2){return rtWire2.requestFrom(adr,qnt,(uint8_t)sendStop);};
#endif	
//-------------------------------------------------------------------------------------------------	
if(bus || qnt || adr || sendStop) {return 0;}		//--For blocking warnings from C++;
return 0;}


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
//										4.INIT bus i2c;
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//						        	4.1 INIT bus i2c as MASTER
//=================================================================================================
uint8_t  RT_HW_BASE:: i2cBegin(uint8_t bus){
//-------------------------------------------------------------------------------------------------
i2cClearCodeErr(bus);								//--Cleaning code error;
if(i2cGetStatus(bus)==1){return 1;} 				//--Exit if bus is already Master;
if(i2cSetBusy(bus)==0)	{return 0;}					//--Bus capture or exit if the bus is busy;
//=================================================================================================
//									4.1.1.STM32
//=================================================================================================
#if defined(RT_HW_CORE_STM32) 
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_CREATED_I2C0)	
if(bus==0){
rtWire0.setSDA(i2cGetPinSDA(bus));					//--Setting pins SDA;		
rtWire0.setSCL(i2cGetPinSCL(bus));					//--Setting pins SCL;		
rtWire0.begin();									//--Init as Master bus=0;
rtWire0.setClock(i2cGetSpeed(bus));;				//--Setting speed default;	
i2cSetStatus(1,bus);								//--Setting status=Master;
i2cClearBusy(bus); 									//--Bus release; 
return 1;}											//--Successful Exit bus=0;
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_CREATED_I2C1)	
if(bus==1){
rtWire1.setSDA(i2cGetPinSDA(bus));					//--Setting pins SDA;		
rtWire1.setSCL(i2cGetPinSCL(bus));					//--Setting pins SCL;		
rtWire1.begin();									//--Init as Master bus=1;
rtWire1.setClock(i2cGetSpeed(bus));;				//--Setting speed default;	
i2cSetStatus(1,bus);								//--Setting status=Master;
i2cClearBusy(bus); 									//--Bus release; 
return 1;}											//--Successful Exit bus=1;
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_CREATED_I2C2)	
if(bus==0){
rtWire2.setSDA(i2cGetPinSDA(bus));					//--Setting pins SDA;		
rtWire2.setSCL(i2cGetPinSCL(bus));					//--Setting pins SCL;		
rtWire2.begin();									//--Init as Master bus=2;
rtWire2.setClock(i2cGetSpeed(bus));;				//--Setting speed default;	
i2cSetStatus(1,bus);								//--Setting status=Master;
i2cClearBusy(bus); 									//--Bus release; 
return 1;}											//--Successful Exit bus=2;
#endif
//-------------------------------------------------------------------------------------------------
i2cSetCodeErr(70,bus);								//--Setting error "No bus";
i2cClearBusy(bus); 									//--Bus release;
return 1;											//--Exit by mistake;
#endif
//=================================================================================================
//									4.1.2.RP2040
//=================================================================================================
#if defined(RT_HW_CORE_RP2040) 
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_CREATED_I2C0)	
if(bus==0){
rtWire0.setSDA(i2cGetPinSDA(bus));					//--Setting pins SDA;	
rtWire0.setSCL(i2cGetPinSCL(bus));					//--Setting pins SCL;			
rtWire0.begin();									//--Init as Master bus=0;
rtWire0.setClock(i2cGetSpeed(bus));;				//--Setting speed default;	
i2cSetStatus(1,bus);								//--Setting status=Master;
i2cClearBusy(bus);									//--Bus release; 
return 1;}											//--Successful Exit bus=0;
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_CREATED_I2C1)	
if(bus==1){
rtWire1.setSDA(i2cGetPinSDA(bus));					//--Setting pins SDA;		
rtWire1.setSCL(i2cGetPinSCL(bus));					//--Setting pins SCL;			
rtWire1.begin();									//--Init as Master bus=1;
rtWire1.setClock(i2cGetSpeed(bus));;				//--Setting speed default;	
i2cSetStatus(1,bus);								//--Setting status=Master;
i2cClearBusy(bus); 									//--Bus release; 
return 1;}											//--Successful Exit bus=1;
#endif
//-------------------------------------------------------------------------------------------------
i2cSetCodeErr(70,bus);								//--Setting error "No bus";
i2cClearBusy(bus); 									//--Bus release;
return 1;											//--Exit by mistake;
#endif
//=================================================================================================
//									4.1.3.ESP32
//=================================================================================================
#if defined(RT_HW_CORE_ESP32) 
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_CREATED_I2C0)
if(bus==0){
rtWire0.setPins(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//-Setting pins SDA, SCL;
rtWire0.begin();									//--Init as Master bus=0;	
rtWire0.setClock(i2cGetSpeed(bus));;				//--Setting speed default;	
rtWire0.setTimeOut(50); 							//--Setting time limit for one transaction;
i2cSetStatus(1,bus);								//--Setting status=Master;
i2cClearBusy(bus);									//--Bus release;
return 1;}											//--Successful Exit bus=0;
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_CREATED_I2C1)
if(bus==1){
rtWire1.setPins(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//-Setting pins SDA, SCL;
rtWire1.begin();									//--Init as Master bus=1;
rtWire1.setClock(i2cGetSpeed(bus));;				//--Setting speed default;		
rtWire1.setTimeOut(50); 							//--Setting time limit for one transaction;	
i2cSetStatus(1,bus);								//--Setting status=Master;
i2cClearBusy(bus);									//--Bus release;
return 1;}											//--Successful Exit bus=1;
#endif
//-------------------------------------------------------------------------------------------------
i2cSetCodeErr(70,bus);								//--Setting error "No bus";
i2cClearBusy(bus); 									//--Bus release;
return 1;											//--Exit by mistake; 									//--Bus release;
#endif
//=================================================================================================
//									4.1.2.ESP8266
//=================================================================================================
#if defined(RT_HW_CORE_ESP8266)
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_CREATED_I2C0)	
if(bus==0){
rtWire0.begin(i2cGetPinSDA(bus),i2cGetPinSCL(bus));	//--Init as Master bus=0 with setting pins SDA, SCL;
rtWire0.setClock(i2cGetSpeed(bus));;					//--Setting speed default;	
rtWire0.setClockStretchLimit(260);						//--Setting time limit for one transaction;
i2cSetStatus(1,bus);								//--Setting status=Master;
i2cClearBusy(bus); 									//--Bus release;
return 1;}											//--Successful Exit bus=0;
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_CREATED_I2C1)	
if(bus==1){
rtWire1.begin(i2cGetPinSDA(bus),i2cGetPinSCL(bus));	//--Init as Master bus=0 with setting pins SDA, SCL;
rtWire1.setClock(i2cGetSpeed(bus));					//--Setting speed default;	
rtWire1.setClockStretchLimit(260);					//--Setting time limit for one transaction;
i2cSetStatus(1,bus);								//--Setting status=Master;
i2cClearBusy(bus); 									//--Bus release;
return 1;}											//--Successful Exit bus=1;
#endif
//-------------------------------------------------------------------------------------------------
i2cSetCodeErr(70,bus);								//--Setting error "No bus";
i2cClearBusy(bus); 									//--Bus release;
return 1;											//--Exit by mistake;
#endif

//=================================================================================================
//									4.1.1.AVR,SAM3X,SAMD21,ANON
//=================================================================================================
#if defined(RT_HW_CORE_AVR) || defined(RT_HW_CORE_SAM3X) || defined(RT_HW_CORE_SAMD21) || defined(RT_HW_CORE_ANON)
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_CREATED_I2C0)	
if(bus==0){
rtWire0.begin(); 									//--Init as Master bus=0;
i2cSetStatus(1,bus);								//--Setting status=Master;
i2cClearBusy(bus);									//--Bus release;
return  1;}											//--Successful Exit bus=0;
#endif
//-------------------------------------------------------------------------------------------------
i2cSetCodeErr(70,bus);								//--Setting error "No bus";
i2cClearBusy(bus); 									//--Bus release;
return 1;											//--Exit by mistake;
#endif
//=================================================================================================
//									4.1.6. No CORE									
//=================================================================================================
i2cSetCodeErr(65,bus);								//--Setting codeErr=No CORE;
i2cClearBusy(bus);									//--Bus release; 
return 1;											//--Exit by mistake;
}; 
//=================================================================================================




//==================================================================================================




/*
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C0)
if(bus==0){									
#if defined(RT_HW_CORE_AVR) || defined(RT_HW_CORE_SAM) || defined(RT_HW_CORE_SAMD)
Wire.begin();							//--Инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;
i2cSetStatus(1,bus); 					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif
#if defined(RT_HW_CORE_ESP8266)
Wire.begin(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//--Установка номеров пинов SDA,SCL и инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
Wire.setClockStretchLimit(260);			//--Ограничение лимита по времени;
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif	
#if defined(RT_HW_CORE_ESP32)			 													
Wire.setPins(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//--Настройка пинов SDA,SCL;
Wire.begin();							//--Инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;		
Wire.setTimeOut(50); 					//--По умолчанию  тайм-аут на транзакцию =50ms;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif		//-------------------------------------------------------------------------------------
#if defined(RT_HW_CORE_STM32) || defined(RT_HW_CORE_RP2040)
//consoleTest(String(F("init i2c")),(String)RT_HW_BOARD_SELECT);
Wire.setSDA(i2cGetPinSDA(bus));			//--Установка номера пина SDA;	
Wire.setSCL(i2cGetPinSCL(bus));			//--Установка номера пина SCL;		
Wire.begin();							//--Инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif
#if defined(RT_HW_CORE_ANON)
Wire.begin();							//--Инициализация как Master;
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif
i2cSetStatus(5,bus); codeErr=5; 		//--Выход с установкой неопределенного статуса;
i2cClearBusy(bus); return 1;}	//++++Освобождение шины и выход------------------------------------
#endif	//--END #if defined(RT_HW_CREATED_WIRE0)
//=================================================================================================
#if defined(RT_HW_PERMIT_I2C1)
if(bus==1){									
#if defined(RT_HW_CORE_SAM)
rtWire1.begin();							//--Инициализация как Master;
rtWire1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход-------------------------------------
#endif
#if defined(RT_HW_CORE_ESP32)			 													
rtWire1.setPins(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//--Настройка пинов SDA,SCL;
rtWire1.begin();							//--Инициализация как Master;
rtWire1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;			
rtWire1.setTimeOut(50); 					//--По умолчанию  тайм-аут на транзакцию =50ms;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход-------------------------------------
#endif
#if defined(RT_HW_CORE_STM32) || defined(RT_HW_CORE_RP2040) 
rtWire1.setSDA(i2cGetPinSDA(bus));		//--Установка номера пина SDA;	
rtWire1.setSCL(i2cGetPinSCL(bus));		//--Установка номера пина SCL;			
rtWire1.begin();							//--Инициализация как Master;
rtWire1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход-------------------------------------
#endif
LinkI2C1.status=5; codeErr=5;			//--Выход с установкой неопределенного статуса; 
i2cClearBusy(bus); return 1;}	//++++Освобождение шины и выход-------------------------------------	
#endif	//--END #if defined(RT_HW_RT_HW_PERMIT_I2C1)
#if defined(RT_HW_PERMIT_I2C2)
if(bus==2){									
#if defined(RT_HW_CORE_STM32) || defined(RT_HW_CORE_RP2040)  
rtWire2.setSDA(i2cGetPinSDA(bus));		//--Установка номера пина SDA;	
rtWire2.setSCL(i2cGetPinSCL(bus));		//--Установка номера пина SCL;			
rtWire2.begin();							//--Инициализация как Master;
rtWire2.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход-------------------------------------
#endif
LinkI2C2.status=5; codeErr=5;			//--Выход с установкой неопределенного статуса;
i2cClearBusy(bus); return 1;}	//++++Освобождение шины и выход-------------------------------------
//====END if(bus==2)---------------------------------------------------
#endif	//--END #if defined(RT_HW_CREATED_WIRE2)


*/




//=================================================================================================
//						        	4.1. i2cBegin.Master
//=================================================================================================
/*
uint8_t  RT_HW_BASE:: i2cBegin(uint8_t bus){
//-------------------------------------------------------------------------------------------------
codeErr=0;											//--Очистка кода ошибки;
if(i2cCheckBus(bus)==0)	{codeErr=65; return 1;}	//--Выход, если недопустиый номер шины;
if(i2cGetStatus(bus)==1){            return 1;} //--Выход, если шина уже Master;
if(i2cSetBusy(bus)==0)	{            return 0;}	//--Захват шины или выход если шина занята;
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C0)
if(bus==0){									
#if defined(RT_HW_CORE_AVR) || defined(RT_HW_CORE_SAM) || defined(RT_HW_CORE_SAMD)
Wire.begin();							//--Инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;
i2cSetStatus(1,bus); 					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif
#if defined(RT_HW_CORE_ESP8266)
Wire.begin(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//--Установка номеров пинов SDA,SCL и инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
Wire.setClockStretchLimit(260);			//--Ограничение лимита по времени;
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif	
#if defined(RT_HW_CORE_ESP32)			 													
Wire.setPins(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//--Настройка пинов SDA,SCL;
Wire.begin();							//--Инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;		
Wire.setTimeOut(50); 					//--По умолчанию  тайм-аут на транзакцию =50ms;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif		//-------------------------------------------------------------------------------------
#if defined(RT_HW_CORE_STM32) || defined(RT_HW_CORE_RP2040)
//consoleTest(String(F("init i2c")),(String)RT_HW_BOARD_SELECT);
Wire.setSDA(i2cGetPinSDA(bus));			//--Установка номера пина SDA;	
Wire.setSCL(i2cGetPinSCL(bus));			//--Установка номера пина SCL;		
Wire.begin();							//--Инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif
#if defined(RT_HW_CORE_ANON)
Wire.begin();							//--Инициализация как Master;
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif
i2cSetStatus(5,bus); codeErr=5; 		//--Выход с установкой неопределенного статуса;
i2cClearBusy(bus); return 1;}	//++++Освобождение шины и выход------------------------------------
#endif	//--END #if defined(RT_HW_CREATED_WIRE0)
//=================================================================================================
#if defined(RT_HW_PERMIT_I2C1)
if(bus==1){									
#if defined(RT_HW_CORE_SAM)
rtWire1.begin();							//--Инициализация как Master;
rtWire1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход-------------------------------------
#endif
#if defined(RT_HW_CORE_ESP32)			 													
rtWire1.setPins(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//--Настройка пинов SDA,SCL;
rtWire1.begin();							//--Инициализация как Master;
rtWire1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;			
rtWire1.setTimeOut(50); 					//--По умолчанию  тайм-аут на транзакцию =50ms;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход-------------------------------------
#endif
#if defined(RT_HW_CORE_STM32) || defined(RT_HW_CORE_RP2040) 
rtWire1.setSDA(i2cGetPinSDA(bus));		//--Установка номера пина SDA;	
rtWire1.setSCL(i2cGetPinSCL(bus));		//--Установка номера пина SCL;			
rtWire1.begin();							//--Инициализация как Master;
rtWire1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход-------------------------------------
#endif
LinkI2C1.status=5; codeErr=5;			//--Выход с установкой неопределенного статуса; 
i2cClearBusy(bus); return 1;}	//++++Освобождение шины и выход-------------------------------------	
#endif	//--END #if defined(RT_HW_RT_HW_PERMIT_I2C1)
#if defined(RT_HW_PERMIT_I2C2)
if(bus==2){									
#if defined(RT_HW_CORE_STM32) || defined(RT_HW_CORE_RP2040)  
rtWire2.setSDA(i2cGetPinSDA(bus));		//--Установка номера пина SDA;	
rtWire2.setSCL(i2cGetPinSCL(bus));		//--Установка номера пина SCL;			
rtWire2.begin();							//--Инициализация как Master;
rtWire2.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход-------------------------------------
#endif
LinkI2C2.status=5; codeErr=5;			//--Выход с установкой неопределенного статуса;
i2cClearBusy(bus); return 1;}	//++++Освобождение шины и выход-------------------------------------
//====END if(bus==2)---------------------------------------------------
#endif	//--END #if defined(RT_HW_CREATED_WIRE2)
codeErr=7;								//--Выход с установкой ошибки "Нет шины"
i2cClearBusy(bus); return 1;}; 
*/




/*
//=================================================================================================
//						        НАСТРОЙКА ШИНЫ i2c.Master
//=================================================================================================
uint8_t  RT_HW_BASE:: i2cBeginMaster(uint8_t &codeErr, uint8_t bus){
//-------------------------------------------------------------------------------------------------
codeErr=0;											//--Очистка кода ошибки;
if(i2cCheckBus(bus)==0)	{codeErr=65; return 1;}	//--Выход, если недопустиый номер шины;
if(i2cGetStatus(bus)==1){            return 1;} //--Выход, если шина уже Master;
if(i2cSetBusy(bus)==0)	{            return 0;}	//--Захват шины или выход если шина занята;
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_I2C0)
if(bus==0){									
#if defined(RT_HW_CORE_AVR) || defined(RT_HW_CORE_SAM) || defined(RT_HW_CORE_SAMD)
Wire.begin();							//--Инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;
i2cSetStatus(1,bus); 					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif
#if defined(RT_HW_CORE_ESP8266)
Wire.begin(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//--Установка номеров пинов SDA,SCL и инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
Wire.setClockStretchLimit(260);			//--Ограничение лимита по времени;
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif	
#if defined(RT_HW_CORE_ESP32)			 													
Wire.setPins(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//--Настройка пинов SDA,SCL;
Wire.begin();							//--Инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;		
Wire.setTimeOut(50); 					//--По умолчанию  тайм-аут на транзакцию =50ms;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif		//-------------------------------------------------------------------------------------
#if defined(RT_HW_CORE_STM32) || defined(RT_HW_CORE_RP2040)
//consoleTest(String(F("init i2c")),(String)RT_HW_BOARD_SELECT);
Wire.setSDA(i2cGetPinSDA(bus));			//--Установка номера пина SDA;	
Wire.setSCL(i2cGetPinSCL(bus));			//--Установка номера пина SCL;		
Wire.begin();							//--Инициализация как Master;
Wire.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif
#if defined(RT_HW_CORE_ANON)
Wire.begin();							//--Инициализация как Master;
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход------------------------------------
#endif
i2cSetStatus(5,bus); codeErr=5; 		//--Выход с установкой неопределенного статуса;
i2cClearBusy(bus); return 1;}	//++++Освобождение шины и выход------------------------------------
#endif	//--END #if defined(RT_HW_CREATED_WIRE0)
//=================================================================================================
#if defined(RT_HW_PERMIT_I2C1)
if(bus==1){									
#if defined(RT_HW_CORE_SAM)
rtWire1.begin();							//--Инициализация как Master;
rtWire1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход-------------------------------------
#endif
#if defined(RT_HW_CORE_ESP32)			 													
rtWire1.setPins(i2cGetPinSDA(bus),i2cGetPinSCL(bus));//--Настройка пинов SDA,SCL;
rtWire1.begin();							//--Инициализация как Master;
rtWire1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;			
rtWire1.setTimeOut(50); 					//--По умолчанию  тайм-аут на транзакцию =50ms;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход-------------------------------------
#endif
#if defined(RT_HW_CORE_STM32) || defined(RT_HW_CORE_RP2040) 
rtWire1.setSDA(i2cGetPinSDA(bus));		//--Установка номера пина SDA;	
rtWire1.setSCL(i2cGetPinSCL(bus));		//--Установка номера пина SCL;			
rtWire1.begin();							//--Инициализация как Master;
rtWire1.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход-------------------------------------
#endif
LinkI2C1.status=5; codeErr=5;			//--Выход с установкой неопределенного статуса; 
i2cClearBusy(bus); return 1;}	//++++Освобождение шины и выход-------------------------------------	
#endif	//--END #if defined(RT_HW_RT_HW_PERMIT_I2C1)
#if defined(RT_HW_PERMIT_I2C2)
if(bus==2){									
#if defined(RT_HW_CORE_STM32) || defined(RT_HW_CORE_RP2040)  
rtWire2.setSDA(i2cGetPinSDA(bus));		//--Установка номера пина SDA;	
rtWire2.setSCL(i2cGetPinSCL(bus));		//--Установка номера пина SCL;			
rtWire2.begin();							//--Инициализация как Master;
rtWire2.setClock(i2cGetSpeed(bus));;		//--Установка частоты шины;	
i2cSetStatus(1,bus);					//--Установка статуса Master и успешный выход;
i2cClearBusy(bus); return 1;	//++++Освобождение шины и выход-------------------------------------
#endif
LinkI2C2.status=5; codeErr=5;			//--Выход с установкой неопределенного статуса;
i2cClearBusy(bus); return 1;}	//++++Освобождение шины и выход-------------------------------------
//====END if(bus==2)---------------------------------------------------
#endif	//--END #if defined(RT_HW_CREATED_WIRE2)
codeErr=7;								//--Выход с установкой ошибки "Нет шины"
i2cClearBusy(bus); return 1;}; 
*/


//=================================================================================================
//								Настройка шины как Slave
//=================================================================================================
//void    RT_HW_BASE:: i2cBeginSlave(uint8_t bus){
//if(i2cGetStatus(bus)==1) {return;};
//};
//--END i2cBegin()
//=================================================================================================




//=================================================================================================
//								1.CREATING ADDITIONAL I2C OBJECTS UNDER MCU TYPE
//=================================================================================================
//#if defined(RT_HW_PERMIT_I2C0) 			//--The object is declared by default for all architectures;
//#define RT_HW_CREATED_I2C0
//#endif
//=================================================================================================
//								1.1.CREATING AN OBJECT I2C for ESP8266
//=================================================================================================
//------------------------------------------1.1.1.I2C1.ESP8266-------------------------------------
//#if defined(RT_HW_PERMIT_I2C1)  && defined(RT_HW_CORE_ESP8266)	
//#if !defined(NO_GLOBAL_INSTANCES) && !defined(NO_GLOBAL_TWOWIRE)
//#define RT_HW_CREATED_I2C1
//TwoWire rtWire1;
//#endif
//#endif
//=================================================================================================
//								1.2.CREATING AN OBJECT I2C for ESP32
//=================================================================================================
//------------------------------------------1.2.1.I2C1.ESP32---------------------------------------
//#if defined(RT_HW_PERMIT_I2C1) && defined(RT_HW_CORE_ESP32)	
//#define RT_HW_CREATED_I2C1
//TwoWire rtWire1(1);
//#endif
//=================================================================================================
//								1.3.CREATING AN OBJECT I2C for RP2040
//=================================================================================================
//------------------------------------------1.3.1.I2C1.RP2040--------------------------------------
//#if defined(RT_HW_PERMIT_I2C1) && defined(RT_HW_CORE_RP2040)
//#define RT_HW_CREATED_I2C1
//TwoWire rtWire1(i2c1, RT_HW_I2C1_SDA, RT_HW_I2C1_SCL);
//#endif
//=================================================================================================
//								1.4.CREATING AN OBJECT I2C for STM32
//=================================================================================================
//------------------------------------------1.4.1.I2C1.STM32---------------------------------------
//#if defined(RT_HW_PERMIT_I2C1)  && defined(RT_HW_CORE_STM32)
//#define RT_HW_CREATED_I2C1
//TwoWire rtWire1(RT_HW_I2C1_SDA, RT_HW_I2C1_SCL);
//#endif
//------------------------------------------1.4.2.I2C2.STM32---------------------------------------
//#if defined(RT_HW_PERMIT_I2C2)  && defined(RT_HW_CORE_STM32) 
//#define RT_HW_CREATED_I2C2
//TwoWire rtWire2(RT_HW_I2C2_SDA, RT_HW_I2C2_SCL);
//#endif
//=================================================================================================

