//#################################################################################################
//
//                        				API.h для SPI
//ecoins@mail.ru 26.04.2023г.
//=================================================================================================
// AVR  	- one bus;
// ESP8266  - one bus;
// ESP32S   - two bus; 
// SAM3     - one bus;
// STM32    - 1-3 bus;
// Raspberry Pi Pico -two bus.
//=================================================================================================
//#################################################################################################

//=================================================================================================
#define  RT_HW_LINK_SPI		//---FUNCTION CONNECTION DEFINER SPI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//									0.STRUCTURE FOR SPI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_STRUCT_SPI {
uint8_t  custom=0;				//--1.Setting option;
uint8_t  status=0;				//--2.State bus: no init=0; Master=1; Slave=2;
uint8_t  codeErr=0;				//--3.Code error; 	
uint8_t  busy=0;  				//--4.busy bus;
//-------------------------------------------------------------------------------------------------
uint8_t  sck=255; 				//--5.pin SCK;
uint8_t  mosi=255; 				//--6.pin MOSI;
uint8_t  miso=255; 				//--7.pin MISO;	
uint8_t  ss=0; 	    			//--8.pin SS/CS: for Slave=ss, for Master=сs;
//-------------------------------------------------------------------------------------------------
uint8_t  idx=0;					//--9.index;
uint8_t  step;					//--10.Step for inner function;
uint8_t  bf8;
uint32_t timeBegin;				//--11.Time for inner function;
//-------------------------------------------------------------------------------------------------
uint32_t speed=RT_HW_SPI_SPEED;	//--12.Speed bus;	
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//									1.OBJECT DATA for SPI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//=================================================================================================
//									1.0.CREATED LinkSPIx
//=================================================================================================
#if defined(RT_HW_PERMIT_SPI0)							//++++DATA for SPI0------------------------				  
RT_HW_STRUCT_SPI 	LinkSPI0;
#endif	//-----------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_SPI1)							//++++DATA for SPI1------------------------		  
RT_HW_STRUCT_SPI 	LinkSPI1;
#endif	//-----------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_SPI2)							//++++DATA for SPI2------------------------			  
RT_HW_STRUCT_SPI 	LinkSPI2;						
#endif	//----------------------------------------------------------------------------------------- 


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                          		2.InitParam
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//									2.1.Set Pins
//=================================================================================================
void  spiSetPins(uint8_t sck, uint8_t miso, uint8_t mosi, uint8_t bus){ 
//=================================================================================================
#if defined(RT_HW_CORE_AVR) || defined(RT_HW_CORE_SAM)
#if defined(RT_HW_PERMIT_SPI0)
if(bus==0){if(sck || miso || mosi || bus){return;}} //--To block warning C++;
return;
#endif
#endif
//=================================================================================================
#if defined(RT_HW_CORE_ESP8266)
#if defined(RT_HW_PERMIT_SPI0)
if(bus==0){
if((sck==miso) || (sck==mosi) || (miso==mosi)) {spiSetCodeErr(60,bus);   return;}
if(miso==16)								   {spiSetCodeErr(61,bus);   return;}
spiSetPinSCK(sck,bus); spiSetPinMISO(miso,bus); spiSetPinMOSI(mosi,bus); return;
}
#endif
//-------------------------------------------------------------------------------------------------
#endif
//=================================================================================================
#if defined(RT_HW_CORE_ESP32)
#if defined(RT_HW_PERMIT_SPI0)
if(bus==0){
if((sck==miso) || (sck==mosi) || (miso==mosi)) {spiSetCodeErr(60,bus);   return;}
if(sck >=34) 								   {spiSetCodeErr(61,bus);   return;}
if(mosi>=34)                                   {spiSetCodeErr(62,bus);   return;}
spiSetPinSCK(sck,bus); spiSetPinMISO(miso,bus); spiSetPinMOSI(mosi,bus); return;
}
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_SPI1)
if(bus==1){
if((sck==miso) || (sck==mosi) || (miso==mosi)) {spiSetCodeErr(60,bus);   return;}
if(sck >=34) 								   {spiSetCodeErr(61,bus);   return;}
if(mosi>=34)                                   {spiSetCodeErr(62,bus);   return;}
spiSetPinSCK(sck,bus); spiSetPinMISO(miso,bus); spiSetPinMOSI(mosi,bus); return;
}
#endif
//-------------------------------------------------------------------------------------------------
#endif
//=================================================================================================
#if defined(RT_HW_CORE_RP2040)
spiSetBuff(0,bus);
//++++SPI (SCK,MISO,MOSI) -  2, 4, 5;  6, 8, 7; 18,16;+++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_PERMIT_SPI0)
if(bus==0){
if(!spiGetBuff(bus)){if((sck== 0) && (miso== 1) && (mosi== 5)){spiSetBuff(1,bus);}}
if(!spiGetBuff(bus)){if((sck== 6) && (miso== 7) && (mosi== 8)){spiSetBuff(1,bus);}}
if(!spiGetBuff(bus)){if((sck==18) && (miso==16) && (mosi==19)){spiSetBuff(1,bus);}}
if( i2cGetBuff(bus)){spiSetPinSCK(sck,bus); spiSetPinMISO(miso,bus); spiSetPinMOSI(mosi,bus);}
return;}
#endif
//++++SPI1(SCK,MISO,MOSI) - 14, 8,15; 10,12,11+++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_PERMIT_SPI1)
if(bus==1){
if(!spiGetBuff(bus)){if((sck==14) && (miso== 8) && (mosi==15)){spiSetBuff(1,bus);}}
if(!spiGetBuff(bus)){if((sck==10) && (miso==12) && (mosi==11)){spiSetBuff(1,bus);}}
if( i2cGetBuff(bus)){spiSetPinSCK(sck,bus); spiSetPinMISO(miso,bus); spiSetPinMOSI(mosi,bus);}
return;}
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#endif
//=================================================================================================
#if defined(RT_HW_CORE_STM32)
//++++SPI (SCK,MISO,MOSI) -  2, 4, 5;  6, 8, 7; 18,16;+++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_PERMIT_SPI0)
#endif
//++++SPI1(SCK,MISO,MOSI) - 14, 8,15; 10,12,11+++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_PERMIT_SPI1)
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#endif
//=================================================================================================
if((sck==miso) || (sck==mosi) || (miso==mosi)) {spiSetCodeErr(60,bus);   return;}
if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID, sck))    {spiSetCodeErr(61,bus);   return;}
if(!checkPinPGM(RT_HW_PGM_PIN_DIN_ID,miso))	   {spiSetCodeErr(62,bus);   return;}
if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID,mosi))	   {spiSetCodeErr(63,bus);   return;}
spiSetPinSCK(sck,bus); spiSetPinMISO(miso,bus); spiSetPinMOSI(mosi,bus); return;
};
//=================================================================================================
//						    		2.2.Init Param All.
//=================================================================================================
void  spiInitParamAll(){
for(uint8_t i=0; i<=6; i++){spiInitParam(i);}	
}; 
//=================================================================================================
// 		    						2.3.Init default parameters.
//=================================================================================================
void  spiInitParam(uint8_t bus=0){
#if defined(RT_HW_PERMIT_SPI0)
if(bus==0) {LinkSPI0.busy=LinkSPI0.status=LinkSPI0.codeErr=0;	//--clear busy,status,codeErr;
			LinkSPI0.sck  =readArrPGM(RT_HW_PGM_PIN_SPI_ID,0);	//--set pin SCK; 
			LinkSPI0.miso =readArrPGM(RT_HW_PGM_PIN_SPI_ID,1);	//--set pin MISO;
			LinkSPI0.mosi =readArrPGM(RT_HW_PGM_PIN_SPI_ID,2);	//--set pin MOSI; 
			LinkSPI0.ss   =readArrPGM(RT_HW_PGM_PIN_SPI_ID,3);	//--set pin SS;
			LinkSPI0.speed=RT_HW_SPI_SPEED;}            	    //--set speed;
#endif
#if defined(RT_HW_PERMIT_SPI1)
if(bus==1) {LinkSPI1.busy=LinkSPI1.status=LinkSPI1.codeErr=0;	//--clear busy,status,codeErr;
			LinkSPI1.sck  =readArrPGM(RT_HW_PGM_PIN_SPI_ID,4);	//--set pin SCK; 
			LinkSPI1.miso =readArrPGM(RT_HW_PGM_PIN_SPI_ID,5);	//--set pin MISO;
			LinkSPI1.mosi =readArrPGM(RT_HW_PGM_PIN_SPI_ID,6);	//--set pin MOSI; 
			LinkSPI1.ss   =readArrPGM(RT_HW_PGM_PIN_SPI_ID,7);	//--set pin SS;
			LinkSPI1.speed=RT_HW_SPI_SPEED;}					//--set speed;			
#endif
#if defined(RT_HW_PERMIT_SPI2)
if(bus==2) {LinkSPI2.busy=LinkSPI2.status=LinkSPI2.codeErr=0;	//--clear busy,status,codeErr;
			LinkSPI2.sck  =readArrPGM(RT_HW_PGM_PIN_SPI_ID,8);	//--set pin SCK; 
			LinkSPI2.miso =readArrPGM(RT_HW_PGM_PIN_SPI_ID,9);	//--set pin MISO;
			LinkSPI2.mosi =readArrPGM(RT_HW_PGM_PIN_SPI_ID,10);	//--set pin MOSI; 
			LinkSPI2.ss   =readArrPGM(RT_HW_PGM_PIN_SPI_ID,11);	//--set pin SS;
			LinkSPI2.speed=RT_HW_SPI_SPEED;}				    //--set speed;
#endif
if(bus){return;}; //--For blocking warnings from C++; 
};


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                          		3.INIT DEVICE
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
// 		    						3.1.Set parameters
//=================================================================================================
uint8_t spiSetParam(RT_HW_STRUCT_SPI_DEV &id, uint8_t cs, uint8_t bus, uint32_t speed=0){
if(id.custom==0){id.link=id.status=id.codeErr=id.err=0; id.cs=cs; id.bus=bus; id.speed=speed; id.custom=1;}
return id.custom;};
//=================================================================================================
//									3.2.Init device
//=================================================================================================
void  spiInitDevice(RT_HW_STRUCT_SPI_DEV &id){
id.link=0;
if(id.custom==0){id.codeErr=50; id.link=1; return;} 
if(id.status==1){id.codeErr=0;  id.link=1; return;}
if(id.err==1)   {id.codeErr=51; id.link=1; return;}
id.status=id.codeErr=0;
//------------------------------------------------------------------------------------------------
if(!spiCheckBus(id.bus))	                {id.codeErr=52; id.err=1; id.link=1; return;} //--Check bus;
if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID,id.cs)){id.codeErr=53; id.err=1; id.link=1; return;} //--Check cs;
//-------------------------------------------------------------------------------------------------
if(!spiBegin(id.bus))		{											 		 return;} //--Initting i2c;
if( spiGetStatus(id.bus)!=1){                id.codeErr=54; id.err=1; id.link=1; return;} //--Checking i2c as Master;
//-------------------------------------------------------------------------------------------------
if(id.speed==0){id.speed=RT_HW_SPI_SPEED;}				//--Set speed;
digitalWrite(id.cs,1); pinMode(id.cs,OUTPUT);			//--Init pin cs for OUTPUT;
id.custom=id.status=id.link=1;							//--Setting output parameters;	
};
//=================================================================================================
//									3.3.spiBeginTransaction(id)
//=================================================================================================
void  spiBeginTransaction(RT_HW_STRUCT_SPI_DEV &id){
spiBeginTransaction(id.speed,id.bitOrder,id.dataMode,id.bus);
if(id.t1)      {delayMicroseconds(id.t1);} 
digitalWrite(id.cs,0);
};
//=================================================================================================
//									3.4.spiEndTransaction(id)
//=================================================================================================
void  spiEndTransaction(RT_HW_STRUCT_SPI_DEV &id){
spiEndTransaction(id.bus);
if(id.t2){delayMicroseconds(id.t2);} 
digitalWrite(id.cs,1);
if(id.t3){delayMicroseconds(id.t3);} 
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                          		4.spiTransfer(id) 8-32bit
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//									4.1.spiTransfer(id)
//=================================================================================================
void  spiTransfer(RT_HW_STRUCT_SPI_DEV &id, uint32_t tx, uint32_t &rx, uint8_t qnt=1){
id.link=0; rx=0;
//-------------------------------------------------------------------------------------------------
if(id.status!=1){id.link=1; return;}	//--Checking device and bus setup;
if(!spiSetBusy(id.bus))    {return;};	//--Waiting for Bus Release and Bus capture;  
//-------------------------------------------------------------------------------------------------
if(qnt==0){id.link=1;    	return;}	//--Exit for qnt=0;
if(qnt>4){qnt=4;} 						//--Limitation for qnt;
//-------------------------------------------------------------------------------------------------
spiBeginTransaction(id);				//--Begin transaction;
//-------------------------------------------------------------------------------------------------
for(id.idx=qnt-1; id.idx>=0; id.idx--){rx|=spiTransfer((tx>>(id.idx*8))&0xFF,id.bus)<<(id.idx*8);}
//-------------------------------------------------------------------------------------------------
spiEndTransaction(id);					//--End transaction;
//-------------------------------------------------------------------------------------------------
spiClearBusy(id.bus);					//--Bus release;
//-------------------------------------------------------------------------------------------------
id.link=1;}								//--Successful Exit;
//=================================================================================================
//									4.2.spiWrite(id)
//=================================================================================================
void  spiWrite(RT_HW_STRUCT_SPI_DEV &id, uint32_t tx, uint8_t qnt=1){spiTransfer(id,tx,id.bf32,qnt);};
//=================================================================================================
//									4.3.spiRead(id)
//=================================================================================================
void  spiRead (RT_HW_STRUCT_SPI_DEV &id,              uint8_t qnt=1){spiTransfer(id, 0,id.bf32,qnt);};
//=================================================================================================
//									4.4.spiREAD(id) [Gag - read register]
//=================================================================================================
void  spiREAD(RT_HW_STRUCT_SPI_DEV &id,  uint8_t reg){reg=0; id.link=1;};
//=================================================================================================
//									4.5.spiReadRegs(id) [Gag - read qnt register]
//=================================================================================================
void  spiREADArr(RT_HW_STRUCT_SPI_DEV &id, uint8_t reg, uint8_t *arr, uint8_t qnt=1){reg=0; qnt=arr[0]; id.link=1;};

//=================================================================================================
//									4.6.spiRead74HC165(id)
//=================================================================================================
void  spiRead74HC165(RT_HW_STRUCT_SPI_DEV &id, uint8_t qnt){
id.link=0; id.bf32=0;
//-------------------------------------------------------------------------------------------------
if(id.status!=1){id.link=1; return;}	//--Checking device and bus setup;
if(id.qnt==0)   {id.link=1; return;}	//--Checking qnt;  
if(!spiSetBusy(id.bus))    {return;};	//--Waiting for Bus Release and Bus capture;  
//-------------------------------------------------------------------------------------------------
spiBeginTransaction(id.speed,id.bitOrder,id.dataMode,id.bus);
if(id.t1){delayMicroseconds(id.t1);} digitalWrite(id.cs,0); 
if(id.t2){delayMicroseconds(id.t2);} digitalWrite(id.cs,1);
if(id.t3){delayMicroseconds(id.t3);}
//-------------------------------------------------------------------------------------------------
for(id.idx=qnt-1; id.idx>=0; id.idx--){id.bf32|=(spiTransfer(0,id.bus))<<(id.idx*8);
}
//-------------------------------------------------------------------------------------------------
spiEndTransaction(id.bus);				//--End transaction;
//-------------------------------------------------------------------------------------------------
spiClearBusy(id.bus);					//--Bus release;
//-------------------------------------------------------------------------------------------------
id.link=1;}								//--Successful Exit;
//=================================================================================================

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                          		5.spiTransferArr(id) qnt byte
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//									5.1.spiTransferArr(id)
//=================================================================================================
void  spiTransferArr(RT_HW_STRUCT_SPI_DEV &id, uint8_t *tx, uint8_t *rx, uint8_t qnt){
id.link=0;
//-------------------------------------------------------------------------------------------------
if(id.status!=1){id.link=1; return;}	//--Checking device and bus setup;
if(!spiSetBusy(id.bus))    {return;};	//--Waiting for Bus Release and Bus capture;  
//-------------------------------------------------------------------------------------------------
if(qnt==0){id.link=1;    	return;}	//--Exit for qnt=0;
//-------------------------------------------------------------------------------------------------
spiBeginTransaction(id);				//--Begin transaction;
//-------------------------------------------------------------------------------------------------
for(id.idx=0; id.idx<qnt; id.idx++){rx[id.idx]=spiTransfer(tx[id.idx],id.bus);}
//-------------------------------------------------------------------------------------------------
spiEndTransaction(id);					//--End transaction;
//-------------------------------------------------------------------------------------------------
spiClearBusy(id.bus);					//--Bus release;
//-------------------------------------------------------------------------------------------------
id.link=1;}								//--Successful Exit;
//=================================================================================================
//									5.2.spiWriteArr(id)
//=================================================================================================
void  spiWriteArr(RT_HW_STRUCT_SPI_DEV &id, uint8_t *tx, uint8_t qnt){
id.link=0;
//-------------------------------------------------------------------------------------------------
if(id.status!=1){id.link=1; return;}	//--Checking device and bus setup;
if(!spiSetBusy(id.bus))    {return;};	//--Waiting for Bus Release and Bus capture;  
//-------------------------------------------------------------------------------------------------
if(qnt==0){id.link=1;    	return;}	//--Exit for qnt=0;
//-------------------------------------------------------------------------------------------------
spiBeginTransaction(id);				//--Begin transaction;
//-------------------------------------------------------------------------------------------------
for(id.idx=0; id.idx<qnt; id.idx++){spiTransfer(tx[id.idx],id.bus);}
//-------------------------------------------------------------------------------------------------
spiEndTransaction(id);					//--End transaction;
//-------------------------------------------------------------------------------------------------
spiClearBusy(id.bus);					//--Bus release;
//-------------------------------------------------------------------------------------------------
id.link=1;}								//--Successful Exit;
//=================================================================================================
//									5.3.spiReadArr(id)
//=================================================================================================
void  spiReadArr(RT_HW_STRUCT_SPI_DEV &id, uint8_t *rx, uint8_t qnt){
id.link=0;
//-------------------------------------------------------------------------------------------------
if(id.status!=1){id.link=1; return;}	//--Checking device and bus setup;
if(!spiSetBusy(id.bus))    {return;};	//--Waiting for Bus Release and Bus capture;  
//-------------------------------------------------------------------------------------------------
if(qnt==0){id.link=1;    	return;}	//--Exit for qnt=0;
//-------------------------------------------------------------------------------------------------
spiBeginTransaction(id);				//--Begin transaction;
//-------------------------------------------------------------------------------------------------
for(id.idx=0; id.idx<qnt; id.idx++){rx[id.idx]=spiTransfer(0,id.bus);}
//-------------------------------------------------------------------------------------------------
spiEndTransaction(id);					//--End transaction;
//-------------------------------------------------------------------------------------------------
spiClearBusy(id.bus);					//--Bus release;
//-------------------------------------------------------------------------------------------------
id.link=1;}								//--Successful Exit;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//									10.Functions SPI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void 	 spiBeginTransaction(uint32_t speed, uint8_t bitOrder, uint8_t dataMode, uint8_t bus=0);//++++ speed,        bitOrder,         dataMode;
void 	 spiBeginTransaction(uint32_t speed, uint8_t bus=0);									//++++ speed,default[bitOrder=MSBFIRST,dataMode=MODE0]
void     spiBeginTransaction(uint8_t bus=0);													//++++[speed=default,bitOrder=MSBFIRST,dataMode=MODE0]
void 	 spiEndTransaction  (uint8_t bus=0);													//++++End Transaction;
uint8_t	 spiTransfer        (uint8_t  val, uint8_t bus);										//++++Transfer byte;
uint8_t  spiBegin           (uint8_t bus=0);													//++++Init Master;													
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//									1.Functions preparatory
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//									1.0.1.Check bus
//=================================================================================================
uint8_t	 spiCheckBus(uint8_t bus){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {return 1;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {return 1;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {return 1;}
#endif	
//-------------------------------------
if(bus){return 0;}; //--For blocking warnings from C++;
return 0;};;
//=================================================================================================
//									1.1.Get,Set pin SCK
//=================================================================================================
uint8_t spiGetPinSCK (uint8_t bus=0){						
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {return LinkSPI0.sck;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {return LinkSPI1.sck;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {return LinkSPI2.sck;}
#endif
//-------------------------------------
if(bus){return 255;}; //--For blocking warnings from C++;
return 255;};
//-------------------------------------------------------------------------------------------------
void 	spiSetPinSCK( uint8_t sck, uint8_t bus=0){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {LinkSPI0.sck=sck; return;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {LinkSPI1.sck=sck; return;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {LinkSPI2.sck=sck; return;}
#endif
//-------------------------------------
if(sck || bus){return;}; //--For blocking warnings from C++;
};
//=================================================================================================
//									1.2.Get,Set pin MISO
//=================================================================================================
uint8_t spiGetPinMISO(uint8_t bus=0){	
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {return LinkSPI0.miso;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {return LinkSPI1.miso;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {return LinkSPI2.miso;}
#endif
//-------------------------------------
if(bus){return 255;}; //--For blocking warnings from C++;
return 255;};
//-------------------------------------------------------------------------------------------------
void    spiSetPinMISO (uint8_t miso, uint8_t bus=0){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {LinkSPI0.miso=miso; return;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {LinkSPI1.miso=miso; return;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {LinkSPI2.miso=miso; return;}
#endif
//-------------------------------------
if(miso || bus){return;}; //--For blocking warnings from C++;
};
//=================================================================================================
//									1.3.Get,Set pin MOSI
//=================================================================================================
uint8_t spiGetPinMOSI(uint8_t bus=0){	
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {return LinkSPI0.mosi;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {return LinkSPI1.mosi;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {return LinkSPI2.mosi;}
#endif
//-------------------------------------
if(bus){return 255;}; //--For blocking warnings from C++;
return 255;};
//-------------------------------------------------------------------------------------------------
void    spiSetPinMOSI (uint8_t mosi, uint8_t bus=0){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {LinkSPI0.mosi=mosi; return;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {LinkSPI1.mosi=mosi; return;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {LinkSPI2.mosi=mosi; return;}
#endif
//-------------------------------------
if(mosi || bus){return;}; //--For blocking warnings from C++;
};
//=================================================================================================
//									1.4.Get,Set pin SS
//=================================================================================================
uint8_t spiGetPinSS(uint8_t bus=0){	
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {return LinkSPI0.ss;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {return LinkSPI1.ss;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {return LinkSPI2.ss;}
#endif
//-------------------------------------
if(bus){return 255;}; //--For blocking warnings from C++;
return 255;};
//-------------------------------------------------------------------------------------------------
void    spiSetPinSS (uint8_t ss, uint8_t bus=0){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {LinkSPI0.ss=ss; return;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {LinkSPI1.ss=ss; return;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {LinkSPI2.ss=ss; return;}
#endif
//-------------------------------------
if(ss || bus){return;}; //--For blocking warnings from C++;
};
//=================================================================================================
//									1.5.Get,Set status
//=================================================================================================
uint8_t  spiGetStatus(uint8_t bus){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {return LinkSPI0.status;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {return LinkSPI1.status;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {return LinkSPI2.status;}
#endif
//-------------------------------------
if(bus){return 0;}; //--For blocking warnings from C++;
return 0;};
//-------------------------------------------------------------------------------------------------
void     spiSetStatus(uint8_t var,uint8_t bus){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {LinkSPI0.status=var; return;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {LinkSPI1.status=var; return;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {LinkSPI2.status=var; return;}
#endif
//-------------------------------------
if(var || bus){return;}; //--For blocking warnings from C++;
}
//=================================================================================================
//									1.6.Get,Set,Clear codeErr
//=================================================================================================
uint8_t  spiGetCodeErr(uint8_t bus){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {return LinkSPI0.codeErr;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {return LinkSPI1.codeErr;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {return LinkSPI2.codeErr;}
#endif
//-------------------------------------
if(bus){return 90;}; //--For blocking warnings from C++;
return 90;};
//-------------------------------------------------------------------------------------------------
void     spiSetCodeErr(uint8_t var,uint8_t bus){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {LinkSPI0.codeErr=var; return;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {LinkSPI1.codeErr=var; return;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {LinkSPI2.codeErr=var; return;}
#endif
//-------------------------------------
if(var || bus){return;}; //--For blocking warnings from C++;
};
//-------------------------------------------------------------------------------------------------
void     spiClearCodeErr(uint8_t bus){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {LinkSPI0.codeErr=0; return;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {LinkSPI1.codeErr=0; return;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {LinkSPI2.codeErr=0; return;}
#endif
//-------------------------------------
if(bus){return;}; //--For blocking warnings from C++;
}; 
//=================================================================================================
//									===1.7.1. Get Buff
//=================================================================================================
uint8_t  spiGetBuff(uint8_t bus){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {return LinkSPI0.bf8;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {return LinkSPI1.bf8;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {return LinkSPI2.bf8;}
#endif
//-------------------------------------
if(bus){return 0;}; //--For blocking warnings from C++;
return 0;}
//=================================================================================================
//								 	===1.7.2. Set buff
//=================================================================================================
void    spiSetBuff(uint8_t var, uint8_t bus){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {LinkSPI0.bf8=var; return;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {LinkSPI1.bf8=var; return;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {LinkSPI2.bf8=var; return;}
#endif
//-------------------------------------
if(var || bus){return;}; //--For blocking warnings from C++;
};






//=================================================================================================
//									1.7.Check,Capture,Release busy
//=================================================================================================
uint8_t  spiCheckBusy(uint8_t bus){	
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {return LinkSPI0.busy;} 
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {return LinkSPI1.busy;} 
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {return LinkSPI2.busy;} 
#endif
//-------------------------------------
if(bus){return 0;}; //--For blocking warnings from C++;
return 0;};
//-------------------------------------------------------------------------------------------------
uint8_t  spiSetBusy   (uint8_t bus){	
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {if(LinkSPI0.busy==0){LinkSPI0.busy=1;} if(LinkSPI0.busy==0){LinkSPI0.busy=1;} return LinkSPI0.busy;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==0) {if(LinkSPI1.busy==0){LinkSPI1.busy=1;} if(LinkSPI1.busy==0){LinkSPI1.busy=1;} return LinkSPI1.busy;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==0) {if(LinkSPI2.busy==0){LinkSPI2.busy=1;} if(LinkSPI2.busy==0){LinkSPI2.busy=1;} return LinkSPI2.busy;}
#endif
//-------------------------------------
if(bus){return 0;}; //--For blocking warnings from C++;
return 0;};
//-------------------------------------------------------------------------------------------------
void     spiClearBusy (uint8_t bus){	
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {LinkSPI0.busy=0; return;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {LinkSPI1.busy=0; return;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {LinSPI2.busy=0; return;}
#endif
//-------------------------------------
if(bus){return;}; //--For blocking warnings from C++;
};
//=================================================================================================
//									1.8.Get,Set speed
//=================================================================================================
uint32_t spiGetSpeed(uint8_t bus){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {return LinkSPI0.speed;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {return LinkSPI1.speed;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {return LinkSPI2.speed;}
#endif
//-------------------------------------
if(bus){return RT_HW_SPI_SPEED;} //--For blocking warnings from C++;
return RT_HW_SPI_SPEED;
};
//-------------------------------------------------------------------------------------------------
void  	spiSetSpeed(uint32_t speed, uint8_t bus){
if(speed==0){speed=RT_HW_SPI_SPEED;}
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {LinkSPI0.speed=speed; return;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {LinkSPI1.speed=speed; return;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {LinkSPI2.speed=speed; return;}
#endif
//-------------------------------------
if(speed || bus){return;}; //--For blocking warnings from C++;
};