#include "RT_HW_BASE.h"	 
//#################################################################################################
//
//							 					SPI [.cpp]
//
//#################################################################################################

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							 		1.CREATING AN OBJECT SPI for MCU
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//									1.0. For all CORE
//=================================================================================================
#if defined(RT_HW_PERMIT_SPI0) 			//--The object is declared by default for all architectures;
#define RT_HW_CREATED_SPI0
#endif
//=================================================================================================
//								1.1.CREATING AN OBJECT SPI for ESP8266
//=================================================================================================
//--No SPI1 ESP8266
//=================================================================================================
//								1.2.CREATING AN OBJECT SPI for ESP32
//=================================================================================================
//------------------------------------------1.2.1.SPI1.ESP32---------------------------------------
#if defined(RT_HW_PERMIT_SPI1) && defined(RT_HW_CORE_ESP32)	
#define RT_HW_CREATED_SPI1
SPIClass SPI1(HSPI);
#endif
//=================================================================================================
//								1.3.CREATING AN OBJECT SPI for RP2040
//=================================================================================================
//------------------------------------------1.3.1.SPI1.RP2040---------------------------------------
#if defined(RT_HW_PERMIT_SPI1) && defined(RT_HW_CORE_RP2040)	
#define RT_HW_CREATED_SPI1
#endif
//=================================================================================================
//								1.4.CREATING AN OBJECT SPI for STM32
//=================================================================================================
//------------------------------------------1.4.1.SPI1.STM32---------------------------------------
#if defined(RT_HW_PERMIT_SPI1) && defined(RT_HW_CORE_STM32)
#define RT_HW_CREATED_SPI1
SPIClass SPI1(1);
#endif
//------------------------------------------1.4.2.SPI2.STM32---------------------------------------
#if defined(RT_HW_PERMIT_SPI2) && defined(RT_HW_CORE_STM32)
#define RT_HW_CREATED_SPI2
SPIClass SPI2(1);
#endif
//=================================================================================================

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//									2.FUNCTION begin/endTransaction()
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//									2.1.spiBeginTransaction()
//--default=[speed=default,bitOrder=MSBFIRST,dataMode=MODE0];
//--(speed),default=      [bitOrder=MSBFIRST,dataMode=MODE0];
//=================================================================================================
void    RT_HW_BASE:: spiBeginTransaction(uint8_t bus){spiBeginTransaction(spiGetSpeed(bus),1,0,bus);};	
void    RT_HW_BASE:: spiBeginTransaction(uint32_t speed, uint8_t bus){spiBeginTransaction(speed,1,0,bus);};	
//=================================================================================================
//-------------------------------------------------------------------------------------------------
void    RT_HW_BASE:: spiBeginTransaction(uint32_t speed, uint8_t bitOrder, uint8_t dataMode, uint8_t bus){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {if(bitOrder){SPI.beginTransaction (SPISettings(speed,MSBFIRST,dataMode));}	
else                    {SPI.beginTransaction (SPISettings(speed,LSBFIRST,dataMode));}
return;}						 
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {if(bitOrder){SPI1.beginTransaction (SPISettings(speed,MSBFIRST,dataMode));}	
else                    {SPI1.beginTransaction (SPISettings(speed,LSBFIRST,dataMode));}
return;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {if(bitOrder){SPI2.beginTransaction (SPISettings(speed,MSBFIRST,dataMode));}	
else                    {SPI2.beginTransaction (SPISettings(speed,LSBFIRST,dataMode));}
return;}
#endif
if(speed || bitOrder || dataMode || bus){return;}; //--For blocking warnings from C++;;
};
//=================================================================================================
//									2.2.spiEndTransaction()
//=================================================================================================
void    RT_HW_BASE:: spiEndTransaction  (uint8_t bus){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {SPI.endTransaction(); return;}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {SPI1.endTransaction(); return;}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {SPI2.endTransaction(); return;}
#endif
if(bus){return;}	//--For blocking warnings from C++;
};	
//=================================================================================================


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//									3.FUNCTION transfer() 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//									3.1.spiTransfer() v8
//=================================================================================================
uint8_t  RT_HW_BASE:: spiTransfer   (uint8_t  var, uint8_t bus){
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0) {return SPI.transfer(var);}
#endif
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1) {return SPI1.transfer(var);}
#endif
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==2) {return SPI2.transfer(var);}
#endif
if(var || bus){return 0;}	//--For blocking warnings from C++;;
return 0;};


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//									4.FUNCTION begin/end
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//									4.1.spiBegin();
//=================================================================================================
uint8_t  RT_HW_BASE:: spiBegin(uint8_t bus){
spiClearCodeErr(bus);	//--
if(!spiCheckBus(bus))	 {spiSetCodeErr(65,bus); return 1;}	//--Выход, если недопустимый номер шины;
if( spiGetStatus(bus)==1){            return 1;} 			//--Выход, если шина уже Master;
if(!spiSetBusy(bus))	 {            return 0;}			//--Захват шины или выход еcли шина занята
//=================================================================================================
//									4.1.1.AVR,SAM3X,ANON
//=================================================================================================
#if defined(RT_HW_CORE_AVR) || defined(RT_HW_CORE_SAM3X) || defined(RT_HW_CORE_ANON)
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0){
SPI.begin(); spiSetStatus(1,bus); spiClearBusy(bus); return 1;}	
#endif
//-------------------------------------------------------------------------------------------------
#endif
//=================================================================================================
//									4.1.2.ESP8266
// CORE->SPI.pins(sck=14,miso=12,mosi=13,ss=0)==>HSPI;
//=================================================================================================
#if defined(RT_HW_CORE_ESP8266) 
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0){
SPI.pins(14,12,13,0); 							
SPI.begin(); spiSetStatus(1,bus); spiClearBusy(bus); return 1;};	
#endif
//-------------------------------------------------------------------------------------------------
#endif
//=================================================================================================
//									4.1.3.ESP32
//=================================================================================================
#if defined(RT_HW_CORE_ESP32) 
if(bus==0){
if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID,spiGetPinSCK (bus))) 	     {spiSetCodeErr(61,bus);  spiClearBusy(bus); return 1;}
if(!checkPinPGM(RT_HW_PGM_PIN_DIN_ID,spiGetPinMISO(bus)))        {spiSetCodeErr(62,bus);  spiClearBusy(bus); return 1;}
if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID,spiGetPinMOSI(bus)))	     {spiSetCodeErr(63,bus);  spiClearBusy(bus); return 1;}
SPI.begin(spiGetPinSCK(bus),spiGetPinMISO(bus),spiGetPinMOSI(bus)); spiSetStatus(1,bus);  spiClearBusy(bus); return 1;}
#endif
//=================================================================================================
//									4.1.4.RP2040
//=================================================================================================
#if defined(RT_HW_CORE_RP2040) 
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0){
if(!SPI.setSCK(spiGetPinSCK (bus))) 							{spiSetCodeErr(61,bus);  spiClearBusy(bus); return 1;}							
if(!SPI.setRX (spiGetPinMISO(bus))) 							{spiSetCodeErr(62,bus);  spiClearBusy(bus); return 1;}	
if(!SPI.setTX (spiGetPinMOSI(bus))) 							{spiSetCodeErr(63,bus);  spiClearBusy(bus); return 1;}	
SPI.begin(); 					                                   spiSetStatus(1,bus);  spiClearBusy(bus); return 1;}
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1){
if(!SPI1.setSCK(spiGetPinSCK (bus))) 							{spiSetCodeErr(64,bus);  spiClearBusy(bus); return 1;}							
if(!SPI1.setRX (spiGetPinMISO(bus))) 							{spiSetCodeErr(65,bus);  spiClearBusy(bus); return 1;}	
if(!SPI1.setTX (spiGetPinMOSI(bus))) 							{spiSetCodeErr(66,bus);  spiClearBusy(bus); return 1;}	
SPI1.begin(); 					                                   spiSetStatus(1,bus);  spiClearBusy(bus); return 1;}
#endif
//-------------------------------------------------------------------------------------------------
#endif
//=================================================================================================
//									4.1.5.STM32
//=================================================================================================
#if defined(RT_HW_CORE_STM32) 
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_SPI0)	
if(bus==0){
if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID,spiGetPinSCK (bus))) 	     {spiSetCodeErr(61,bus);  spiClearBusy(bus); return 1;}
if(!checkPinPGM(RT_HW_PGM_PIN_DIN_ID,spiGetPinMISO(bus)))        {spiSetCodeErr(62,bus);  spiClearBusy(bus); return 1;}
if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID,spiGetPinMOSI(bus)))	     {spiSetCodeErr(63,bus);  spiClearBusy(bus); return 1;}
SPI.setSCLK(spiGetPinSCK(bus));   SPI.setMISO(spiGetPinMISO(bus)); SPI.setMOSI(spiGetPinMOSI(bus));
SPI.begin(); spiSetStatus(1,bus);  spiClearBusy(bus); return 1;}	
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_SPI1)	
if(bus==1){
if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID,spiGetPinSCK (bus))) 	     {spiSetCodeErr(64,bus);  spiClearBusy(bus); return 1;}
if(!checkPinPGM(RT_HW_PGM_PIN_DIN_ID,spiGetPinMISO(bus)))        {spiSetCodeErr(65,bus);  spiClearBusy(bus); return 1;}
if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID,spiGetPinMOSI(bus)))	     {spiSetCodeErr(66,bus);  spiClearBusy(bus); return 1;}
SPI1.setSCLK(spiGetPinSCK(bus)); SPI1.setMISO(spiGetPinMISO(bus)); SPI1.setMOSI(spiGetPinMOSI(bus));
SPI1.begin(); spiSetStatus(1,bus);  spiClearBusy(bus); return 1;}	
#endif
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_PERMIT_SPI2)	
if(bus==3){
if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID,spiGetPinSCK (bus))) 	     {spiSetCodeErr(67,bus);  spiClearBusy(bus); return 1;}
if(!checkPinPGM(RT_HW_PGM_PIN_DIN_ID,spiGetPinMISO(bus)))        {spiSetCodeErr(68,bus);  spiClearBusy(bus); return 1;}
if(!checkPinPGM(RT_HW_PGM_PIN_DOT_ID,spiGetPinMOSI(bus)))	     {spiSetCodeErr(69,bus);  spiClearBusy(bus); return 1;}
SPI2.setSCLK(spiGetPinSCK(bus)); SPI2.setMISO(spiGetPinMISO(bus)); SPI2.setMOSI(spiGetPinMOSI(bus));
SPI2.begin(); spiSetStatus(1,bus);  spiClearBusy(bus); return 1;}	
#endif
//-------------------------------------------------------------------------------------------------
#endif
//=================================================================================================
//									4.1.9 MCU xxx
//=================================================================================================
																  spiSetCodeErr(60,bus);  spiClearBusy(bus); return 1;	
//=================================================================================================
return 0;}
//=================================================================================================
